package cn.com.taiji.cql.service.impl;
import cn.com.taiji.common.domain.PageResult;
import cn.com.taiji.common.domain.Result;
import cn.com.taiji.common.enums.CommonConstant;
import cn.com.taiji.cql.service.IECqlService;
import lombok.extern.slf4j.Slf4j;
import org.geotools.api.data.DataStore;
import org.geotools.api.data.DataStoreFinder;
import org.geotools.api.data.Query;
import org.geotools.api.data.SimpleFeatureSource;
import org.geotools.api.feature.FeatureVisitor;
import org.geotools.api.feature.simple.SimpleFeature;
import org.geotools.api.feature.type.AttributeDescriptor;
import org.geotools.api.filter.Filter;
import org.geotools.api.filter.sort.SortBy;
import org.geotools.api.filter.sort.SortOrder;
import org.geotools.data.simple.SimpleFeatureCollection;
import org.geotools.data.simple.SimpleFeatureIterator;
import org.geotools.filter.FilterFactoryImpl;
import org.geotools.filter.SortByImpl;
import org.geotools.filter.text.ecql.ECQL;
import org.geotools.jdbc.JDBCDataStoreFactory;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class ECqlServiceImpl implements IECqlService{

    /**
     * cql 通用查询接口
     * @param mapParams
     * @param cql
     * @return
     */
    @Override
    public PageResult cqlQuery4Mysql(Map<String, Object> mapParams, String cql, Map<String,Object> sortMaps, Map<String, Object> pageParams) {

        List<Map<String, Object>> dataList = new ArrayList<Map<String, Object>>();

        DataStore dataStore = null;

        String host = mapParams.get(JDBCDataStoreFactory.HOST.key).toString();

        String database = mapParams.get(JDBCDataStoreFactory.DATABASE.key).toString();

        //查询记录总数
        Long count =0l;

        //当前页数
        Long pageNumber= CommonConstant.PAGE_NUMBER;

        //一页显示多少条记录
        Long pageSize=CommonConstant.PAGE_SIZE;

        //总页数
        Long pageCount =0l;


        try {

            dataStore = DataStoreFinder.getDataStore(mapParams);

            if (dataStore != null) {

                log.info("系统连接到位于：" + host + "的空间数据库" + database + "成功！");

                //根据表名获取source
                SimpleFeatureSource fSource=dataStore.getFeatureSource(mapParams.get("tableName").toString());


                FilterFactoryImpl filterFactory = new FilterFactoryImpl();



                Query query = new Query();

                if(cql !=null && !"".equals(cql)){

                    Filter filter = ECQL.toFilter(cql);

                    query.setFilter(filter);

                }




                if(sortMaps !=null && !"".equals(sortMaps) && sortMaps.keySet().size() >0){

                    List<SortByImpl> sortByList = new ArrayList<SortByImpl>();

                    for (String sortKey : sortMaps.keySet()) {

                        String sortValue = sortMaps.get(sortKey).toString();

                        if("asc".equals(sortValue.toLowerCase())){

                            SortByImpl sb = new SortByImpl(filterFactory.property(sortKey),SortOrder.ASCENDING);

                            sortByList.add(sb);
                        }else{

                            SortByImpl sb = new SortByImpl(filterFactory.property(sortKey),SortOrder.DESCENDING);

                            sortByList.add(sb);

                        }


                    }

                    query.setSortBy(sortByList.toArray(new SortBy[sortByList.size()]));

                }



                //查询记录总数
                count = Long.valueOf(fSource.getCount(query));

                if(pageParams !=null && !"".equals(pageParams) && pageParams.keySet().size() >0){

                    if(pageParams.containsKey("pageNumber")){

                        pageNumber = Long.valueOf(pageParams.get("pageNumber").toString());

                    }

                    if(pageParams.containsKey("pageSize")){

                        pageSize = Long.valueOf(pageParams.get("pageSize").toString());
                    }

                    int startIndex = pageNumber.intValue()-1 <=0?0:pageNumber.intValue()-1;

                    //设置分页查询参数
                    query.setStartIndex(startIndex*pageSize.intValue());

                    query.setMaxFeatures(pageSize.intValue());

                    //计算总页数
                    if (count%pageSize ==0){

                        pageCount = count/pageSize;

                    }else{

                        pageCount = count/pageSize +1;
                    }
                }



                SimpleFeatureCollection simpleFeatureCollection =fSource.getFeatures(query);

                if(simpleFeatureCollection !=null){

                    // 获取当前矢量数据有哪些属性字段值
                    List<AttributeDescriptor> attributeList = simpleFeatureCollection.getSchema().getAttributeDescriptors();


                    SimpleFeatureIterator simpleFeatureIterator = simpleFeatureCollection.features();

                    while (simpleFeatureIterator.hasNext()){
                        //获取每一个要素
                        SimpleFeature simpleFeature = simpleFeatureIterator.next();

                        Map<String, Object> dataMap = new HashMap<String, Object>();

                        String id = simpleFeature.getID();
                        id = id.substring(id.lastIndexOf(".")+1);

                        dataMap.put("id",id);

                        for(AttributeDescriptor attribute:attributeList){

//                            System.out.println(attribute.getLocalName() + ":" + simpleFeature.getAttribute(attribute.getLocalName()));

                            dataMap.put(attribute.getLocalName(),simpleFeature.getAttribute(attribute.getLocalName()));

                        }

                        dataList.add(dataMap);


                    }

                    simpleFeatureIterator.close();


                }

            } else {

                log.info("系统连接到位于：" + host + "的空间数据库" + database + "失败！请检查相关参数");

            }

        }catch (Exception e) {

            e.printStackTrace();
            log.error("系统连接到位于：" + mapParams.get(JDBCDataStoreFactory.HOST.key) + "的空间数据库" + mapParams.get(JDBCDataStoreFactory.DATABASE.key) + "失败！请检查相关参数");

        }finally {

            if(dataStore !=null){

                dataStore.dispose();

            }
        }

        int respCode = 0;

        String respMsg = "操作成功";

        if(count == 0){

            respCode = 1;
            respMsg = "操作成功,但没有查找到记录!";

        }

        return PageResult.<List<Map<String, Object>>>builder().datas(dataList).resp_code(respCode).resp_msg(respMsg).pageSize(pageSize).pageNumber(pageNumber).pageCount(pageCount).recordCount(count).build();


    }

    @Override
    public void cqlAggQuery4Mysql(Map<String, Object> mapParams, String cql, FeatureVisitor visitor) {

        String host = mapParams.get(JDBCDataStoreFactory.HOST.key).toString();

        String database = mapParams.get(JDBCDataStoreFactory.DATABASE.key).toString();

        DataStore dataStore = null;

        try{

            dataStore = DataStoreFinder.getDataStore(mapParams);

            if (dataStore != null) {

                log.info("系统连接到位于：" + host + "的空间数据库" + database + "成功！");

                Query query = new Query();

                if(cql !=null && !"".equals(cql)){

                    Filter filter = ECQL.toFilter(cql);

                    query.setFilter(filter);

                }

                //根据表名获取source
                SimpleFeatureSource fSource = dataStore.getFeatureSource(mapParams.get("tableName").toString());

                SimpleFeatureCollection simpleFeatureCollection =fSource.getFeatures(query);

                simpleFeatureCollection.accepts(visitor,null);

            }


        }catch (Exception e) {

            e.printStackTrace();
            log.error("系统连接到位于：" + mapParams.get(JDBCDataStoreFactory.HOST.key) + "的空间数据库" + mapParams.get(JDBCDataStoreFactory.DATABASE.key) + "失败！请检查相关参数");

        }finally {

            if(dataStore !=null){

                dataStore.dispose();

            }
        }
    }
}
